@@ -1,6 +1,7 @@
 from typing import Any, Callable, Dict, List, Optional
 
 import numpy as np
+
 from recon.operations.core import operation
 from recon.types import Example, Span
 
@@ -76,8 +77,8 @@ def substitute_spans(example: Example, span_subs: Dict[Span, str]) -> Example:
 def augment_example(
     example: Example,
     span_f: Callable[[Span, Any], Optional[str]],
-    spans: List[Span] = None,
-    span_label: str = None,
+    spans: Optional[List[Span]] = None,
+    span_label: Optional[str] = None,
     n_augs: int = 1,
     sub_prob: float = 0.5,
     **kwargs: Any,
@@ -97,7 +98,7 @@ def augment_example(
 
         span_subs = {}
         for span in spans_to_sub:
-            res = span_f(span, **kwargs)  #  type: ignore
+            res = span_f(span, **kwargs)  # type: ignore
             if res:
                 span_subs[span] = res
 
@@ -111,7 +112,7 @@ def augment_example(
     return list(augmented_examples)
 
 
-@operation("recon.v1.augment.ent_label_sub", handles_tokens=False)
+@operation("recon.augment.ent_label_sub.v1", handles_tokens=False)
 def ent_label_sub(
     example: Example, label: str, subs: List[str], n_augs: int = 1, sub_prob: float = 0.5
 ) -> List[Example]:
@@ -140,10 +141,12 @@ def ent_label_sub(
             sub = np.random.choice(subs)
         return sub
 
-    return augment_example(example, augmentation, n_augs=n_augs, sub_prob=sub_prob, subs=subs)
+    return augment_example(
+        example, augmentation, span_label=label, n_augs=n_augs, sub_prob=sub_prob, subs=subs
+    )
 
 
-@operation("recon.v1.augment.kb_expansion", factory=True)
+@operation("recon.augment.kb_expansion.v1", factory=True)
 def kb_expansion(
     example: Example,
     preprocessed_outputs: Dict[str, Any] = {},
@@ -151,7 +154,7 @@ def kb_expansion(
     sub_prob: float = 0.5,
 ) -> List[Example]:
 
-    spans_to_aliases_map = preprocessed_outputs["recon.v1.span_aliases"]
+    spans_to_aliases_map = preprocessed_outputs["recon.span_aliases.v1"]
 
     def augmentation(span: Span, spans_to_aliases_map: Dict[int, List[str]]) -> Optional[str]:
         sub = None
@@ -187,7 +190,7 @@ def kb_expansion(
 #             return words[0].replace("_", " ")
 
 
-# @operation("recon.v1.augment.replace_pos_with_synonym", pre=[spacy_pre])
+# @operation("recon.augment.replace_pos_with_synonym.v1", pre=[spacy_pre])
 # def replace_pos_with_synonym(example: Example, pos: str, synonym_f: Callable[[str], str] = get_synonym, preprocessed_outputs={}, n_augs: int = 1):
 
 #     pos_map = {
