@@ -1,8 +1,22 @@
 import warnings
 from collections import Counter, defaultdict
-from typing import Any, Callable, Dict, Iterator, List, Optional, Tuple, Union, TYPE_CHECKING
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Tuple,
+    Union,
+)
 
 from pydantic.error_wrappers import ErrorWrapper
+from tqdm import tqdm
+from wasabi import Printer
+
 from recon.operations import registry as op_registry
 from recon.operations.utils import (
     get_received_operation_data,
@@ -13,15 +27,12 @@ from recon.preprocess import PreProcessor
 from recon.preprocess import registry as pre_registry
 from recon.types import (
     Example,
-    Op,
     OperationResult,
     OperationState,
     OperationStatus,
     Transformation,
     TransformationType,
 )
-from tqdm import tqdm
-from wasabi import Printer
 
 if TYPE_CHECKING:
     from recon import Dataset
@@ -41,12 +52,14 @@ def op_iter(
     Yields:
         Iterator[Tuple[int, Example]]: Tuples of (example hash, example)
     """
-    msg = Printer(no_print=verbose == False, hide_animation=verbose == False)
+    msg = Printer(no_print=not verbose, hide_animation=not verbose)
     preprocessed_outputs: Dict[Example, Dict[str, Any]] = defaultdict(dict)
     for processor in pre:
         msg.info(f"\t=> Running preprocessor {processor.name}")
         processor_outputs = processor(data)
-        for example, output in tqdm(zip(data, processor_outputs), total=len(data), disable=(not verbose), leave=False):
+        for example, output in tqdm(
+            zip(data, processor_outputs), total=len(data), disable=(not verbose), leave=False
+        ):
             preprocessed_outputs[example][processor.name] = output
             example.__setattr__(processor.field, output)
 
@@ -76,13 +89,13 @@ class operation:
         self.factory = factory
         self.augmentation = augmentation
 
-    def __call__(self, op: Op, *args: Any, **kwargs: Any) -> Callable:
+    def __call__(
+        self, op: Callable[..., Union[Example, Iterable[Example], None]]
+    ) -> Callable[..., Union[Example, Iterable[Example], None]]:
         """Decorator for an operation.
-        The first arg is the function being decorated.
-        This function can either operate on a List[Example]
-        and in that case self.batch should be True.
+        The first arg to the op callable needs to be an Example.
 
-        e.g. @operation("recon.v1.some_name", batch=True)
+        e.g. @operation("recon.some_name", batch=True)
 
         Or it should operate on a single example and
         recon will take care of applying it to a full Dataset
@@ -95,29 +108,15 @@ class operation:
         """
         pre: List[PreProcessor] = []
 
-        for pre_name_or_op in self.pre:
-            preprocessor = pre_name_or_op
+        for preprocessor in self.pre:
             if isinstance(preprocessor, str):
-                preprocessor = pre_registry.preprocessors.get(pre_name_or_op)
+                preprocessor = pre_registry.preprocessors.get(preprocessor)
             assert isinstance(preprocessor, PreProcessor)
             pre.append(preprocessor)
 
-        if self.factory:
-            def factory(pre: List[PreProcessor]) -> Operation:
-                return Operation(
-                    self.name,
-                    pre,
-                    op=op,
-                    handles_tokens=self.handles_tokens,
-                    augmentation=self.augmentation
-                )
-
-            op_registry.operation_factories.register(self.name)(factory)
-        else:
-            op_registry.operations.register(self.name)(
-                Operation(self.name, pre, op, self.handles_tokens, augmentation=self.augmentation)
-            )
-
+        op_registry.operations.register(self.name)(
+            Operation(self.name, pre, op, self.handles_tokens, augmentation=self.augmentation)
+        )
         return op
 
 
@@ -150,9 +149,9 @@ class Operation:
         self,
         dataset: "Dataset",
         *args: Any,
-        verbose: Optional[bool] = False,
+        verbose: bool = False,
         initial_state: Optional[OperationState] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> OperationResult:
         """Runs op on a dataset and records the results
 
@@ -175,18 +174,20 @@ class Operation:
         state.args = args
         state.kwargs = kwargs
 
-        def add_example(new_example: Example) -> None:
+        def track_add_example(new_example: Example) -> None:
             state.transformations.append(
                 Transformation(example=hash(new_example), type=TransformationType.EXAMPLE_ADDED)
             )
             dataset.example_store.add(new_example)
 
-        def remove_example(orig_example_hash: int) -> None:
+        def track_remove_example(orig_example_hash: int) -> None:
             state.transformations.append(
-                Transformation(prev_example=orig_example_hash, type=TransformationType.EXAMPLE_REMOVED)
+                Transformation(
+                    prev_example=orig_example_hash, type=TransformationType.EXAMPLE_REMOVED
+                )
             )
 
-        def change_example(orig_example_hash: int, new_example: Example) -> None:
+        def track_change_example(orig_example_hash: int, new_example: Example) -> None:
             state.transformations.append(
                 Transformation(
                     prev_example=orig_example_hash,
@@ -208,10 +209,11 @@ class Operation:
                 "This dataset seems to have preset tokens. "
                 f"Operation: {self.name} is not currently capable of handling tokens and you will "
                 "need to reset tokenization after this operation. "
-                "Applying the `recon.v1.add_tokens` operation after this "
+                "Applying the `recon.add_tokens.v1` operation after this "
                 "operation is complete will get you back to a clean state."
                 # fmt: on
             )
+            state.status = OperationStatus.NEEDS_TOKENIZATION
 
         required_params = get_required_operation_params(self.op)
         received_data = get_received_operation_data(required_params, state)
@@ -234,17 +236,18 @@ class Operation:
 
         state.args = ()
         state.kwargs = values
-        new_data = []
 
+        new_data = []
         with tqdm(total=len(dataset), disable=(not verbose)) as pbar:
-            for orig_example_hash, example, preprocessed_outputs in op_iter(dataset.data, self.pre, verbose=verbose):
+            it = op_iter(dataset.data, self.pre, verbose=verbose)
+            for orig_example_hash, example, preprocessed_outputs in it:
                 if preprocessed_outputs:
                     res = self.op(example, preprocessed_outputs=preprocessed_outputs, **values)
                 else:
                     res = self.op(example, **values)
 
                 if res is None:
-                    remove_example(orig_example_hash)
+                    track_remove_example(orig_example_hash)
                 elif isinstance(res, list):
                     old_example_present = False
                     for new_example in res:
@@ -252,19 +255,18 @@ class Operation:
                         if hash(new_example) == orig_example_hash:
                             old_example_present = True
                         else:
-                            add_example(new_example)
+                            track_add_example(new_example)
                     if not old_example_present:
-                        remove_example(orig_example_hash)
+                        track_remove_example(orig_example_hash)
                 else:
                     assert isinstance(res, Example)
                     new_data.append(res)
                     if hash(res) != orig_example_hash:
-                        change_example(orig_example_hash, res)
+                        track_change_example(orig_example_hash, res)
 
                 pbar.update(1)
 
         transformation_counts = Counter([t.type for t in state.transformations])
-
         state.examples_added = transformation_counts[TransformationType.EXAMPLE_ADDED]
         state.examples_removed = transformation_counts[TransformationType.EXAMPLE_REMOVED]
         state.examples_changed = transformation_counts[TransformationType.EXAMPLE_CHANGED]
