@@ -18,11 +18,11 @@ from recon.dataset import Dataset
 from recon.insights import get_hardest_examples
 from recon.operations.core import operation
 from recon.recognizer import SpacyEntityRecognizer
-from recon.types import Example, HardestExampleV2
+from recon.types import Example, HardestExample, Span
 
 
 @prodigy.recipe(
-    "recon.ner_correct",
+    "recon.ner-correct.v1",
     # fmt: off
     dataset=("Dataset to save annotations to", "positional", None, str),
     spacy_model=("Loadable spaCy model for tokenization or blank:lang (e.g. blank:en)", "positional", None, str),
@@ -32,7 +32,7 @@ from recon.types import Example, HardestExampleV2
     exclude=("Comma-separated list of dataset IDs whose annotations to exclude", "option", "e", split_string),
     # fmt: on
 )
-def ner_correct(
+def recon_ner_correct_v1(
     dataset: str,
     spacy_model: str,
     source: Union[str, Iterable[dict]],
@@ -58,23 +58,7 @@ def ner_correct(
     rec = SpacyEntityRecognizer(nlp)
     hes = get_hardest_examples(rec, stream)
 
-    def stream_from_hardest_examples(hes: List[HardestExampleV2]):
-
-        for he in hes:
-
-            combined = he.reference.copy(deep=True)
-            pred_spans = []
-            for s in he.prediction.spans:
-                span = s.copy(deep=True)
-                span.label = f"{s.label}:PREDICTED"
-                pred_spans.append(span)
-
-            combined.spans = sorted(combined.spans + pred_spans, key=lambda s: s.start)
-
-            task = combined.dict()
-            yield task
-
-    stream = stream_from_hardest_examples(hes)
+    stream = _stream_from_hardest_examples(hes)
 
     def before_db(examples):
         for eg in examples:
@@ -100,17 +84,17 @@ def ner_correct(
             "blocks": [
                 {"view_id": "spans_manual", "overlapping_spans": True},
             ],
-            "javascript": f"""
+            "javascript": """
 
-            const disablePredInteraction = event => {{
+            const disablePredInteraction = event => {
                 let xpath = "//span[contains(text(),':PREDICTED')]";
                 let matchingElement = document.evaluate(xpath, document, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
                 let length = matchingElement.snapshotLength;
 
-                for (var i = 0; i < length; i++) {{
-                    matchingElement.snapshotItem(i).parentElement.parentElement.onclick = function(e) {{ console.log('PRED SPAN CLIECKED'); e.stopImmediatePropagation(); }}
-                }}
-            }}
+                for (var i = 0; i < length; i++) {
+                    matchingElement.snapshotItem(i).parentElement.parentElement.onclick = function(e) { e.stopImmediatePropagation(); }
+                }
+            }
 
             document.addEventListener('prodigyanswer', disablePredInteraction)
             """,
@@ -119,7 +103,7 @@ def ner_correct(
 
 
 @prodigy.recipe(
-    "recon.ner_merge",
+    "recon.ner-merge.v1",
     # fmt: off
     dataset=("Dataset with saved annotations to from recon.ner_correct", "positional", None, str),
     recon_dataset=("Recon dataset name", "positional", None, str),
@@ -128,7 +112,7 @@ def ner_correct(
     exclude=("Comma-separated list of dataset IDs whose annotations to exclude", "option", "e", split_string),
     # fmt: on
 )
-def ner_merge(
+def recon_ner_merge_v1(
     dataset: str,
     recon_dataset: str,
     source: Union[str, Dataset],
@@ -155,7 +139,7 @@ def ner_merge(
     prodigy_texts_to_examples = {e.text: e for e in prodigy_examples}
 
     prev_len = len(dataset)
-    dataset.apply_("recon.v1.prodigy.merge_examples", prodigy_texts_to_examples)
+    dataset.apply_("recon.prodigy.merge_examples", prodigy_texts_to_examples)
     assert len(dataset) == prev_len
 
     if output_dir:
@@ -163,9 +147,26 @@ def ner_merge(
         dataset.to_disk(output_dir)
 
 
-@operation("recon.v1.prodigy.merge_examples")
+@operation("recon.prodigy.merge_examples.v1")
 def merge_examples(example, prodigy_texts_to_examples):
     if example.text in prodigy_texts_to_examples:
         return prodigy_texts_to_examples[example.text]
     else:
         return example
+
+
+def _stream_from_hardest_examples(hes: List[HardestExample]):
+    for he in hes:
+        combined = he.reference.copy(deep=True)
+        annot_spans = [
+            Span(**span.dict(exclude={"source"}), source="ref") for span in he.reference.spans
+        ]
+        pred_spans = [
+            Span(
+                **span.dict(exclude={"source", "label"}), source="pred", label=f"{span.label}:PRED"
+            )
+            for span in he.prediction.spans
+        ]
+        combined.spans = sorted(annot_spans + pred_spans, key=lambda s: s.start)
+        task = combined.dict()
+        yield task
